
#if !defined(_MemoryAccess_h__)
#define _MemoryAccess_h__

#include <limits.h>
#include "NativeInterpreterFloat.h"

namespace nanojit
{
namespace interpreter
{

template<typename T>
inline T endianSwap(T v) {

	int shiftSrc	= (sizeof(T) - 1) * CHAR_BIT;
	int shiftDst	= 0;
	int iMask		= 0xFF;
	T	result		= 0;

	for (unsigned int i = 0; i < sizeof(T); ++i) {
		result |= ((v >> shiftSrc) & iMask) << shiftDst;
		shiftSrc -= 8;
		shiftDst += 8;
	}

	return result;
}


template<typename T>
T loadValue(uintptr_t a_iAddress) {
	return *(reinterpret_cast<T*>(a_iAddress));
}

template<typename T>
T loadValueReversed(uintptr_t a_iAddress) {
	T v = *(reinterpret_cast<T*>(a_iAddress));
	return endianSwap(v);
}

template<>
inline float32_t loadValueReversed<float32_t>(uintptr_t a_iAddress) {

	union IntToFloat {
		uint32_t	asI;
		float32_t	asF;
	};

	IntToFloat cvti2f;
	cvti2f.asI = *(reinterpret_cast<uint32_t*>(a_iAddress));
	cvti2f.asI = endianSwap(cvti2f.asI);
	return cvti2f.asF;
}

template<>
inline float64_t loadValueReversed<float64_t>(uintptr_t a_iAddress) {

	union QuadToDouble {
		uint64_t	asQ;
		float64_t	asD;
	};

	QuadToDouble cvtq2d;
	cvtq2d.asQ = *(reinterpret_cast<uint64_t*>(a_iAddress));
	cvtq2d.asQ = endianSwap(cvtq2d.asQ);
	return cvtq2d.asD;
}


// store

template<typename T>
void storeValue(uintptr_t a_iAddress, T v) {
	*(reinterpret_cast<T*>(a_iAddress)) = v;
}

template<typename T>
void storeValueReversed(uintptr_t a_iAddress, T v) {
	T r = endianSwap(v);
	*(reinterpret_cast<T*>(a_iAddress)) = r;
}


template<>
inline void storeValueReversed<float32_t>(uintptr_t a_iAddress, float32_t v) {

	union IntToFloat {
		uint32_t	asI;
		float32_t	asF;
	};

	IntToFloat cvti2f;
	cvti2f.asF = v;
	cvti2f.asI = endianSwap(cvti2f.asI);
	*(reinterpret_cast<uint32_t*>(a_iAddress)) = cvti2f.asI;
}

template<>
inline void storeValueReversed<float64_t>(uintptr_t a_iAddress, float64_t v) {

	union QuadToDouble {
		uint64_t	asQ;
		float64_t	asD;
	};

	QuadToDouble cvtq2d;
	cvtq2d.asD = v;
	cvtq2d.asQ = endianSwap(cvtq2d.asQ);
	*(reinterpret_cast<uint64_t*>(a_iAddress)) = cvtq2d.asQ;
}


} // namespace interpreter
} // namespace nanojit


#endif // !defined(_MemoryAccess_h__)
