
#ifndef __nanojit_HeapBlock__
#define __nanojit_HeapBlock__

namespace nanojit
{
	class CodeBlock;

	class HeapBlock
	{
		uint8_t*			_buffer;
		size_t				_size;
		CodeBlock*			_lowestCodeBlock;

		/** (only valid for terminator blocks). Set true just before calling
		 * markCodeChunkExec() and false just after markCodeChunkWrite() */
		bool				_isExec;

		HeapBlock*			_next;

	private:
		void initialize(void* buffer, size_t size, HeapBlock* next);
		void createInternalCodeBlock();

	public:
		static HeapBlock* setupHeapBlockFromBuffer(void* buffer, size_t size, HeapBlock* next);

		HeapBlock* next() { return  _next; }

		const HeapBlock* next() const { return  _next; }

		inline void* buffer() { return _buffer; }

		inline size_t size() const { return _size; }

		inline bool isExec() const { return _isExec; }
		inline void markExec() { _isExec = true; }
		inline void markWritable() { _isExec = false; }

		inline const CodeBlock* lowestCodeBlock() const { return _lowestCodeBlock; }
		inline CodeBlock* lowestCodeBlock() { return _lowestCodeBlock; }
	};

} // namespace nanojit

#endif // #ifndef __nanojit_HeapBlock__


