
#if !defined(__nanojit_LirFilter__)
#define __nanojit_LirFilter__

namespace nanojit
{

	class LirFilter
	{
	public:
		LirFilter *in;
		LirFilter(LirFilter *in) : in(in) {}
		virtual ~LirFilter(){}

		// It's crucial that once this reaches the LIR_start at the beginning
		// of the buffer, that it just keeps returning that LIR_start LIns on
		// any subsequent calls.
		virtual LIns* read() {
			return in->read();
		}
		virtual LIns* finalIns() {
			return in->finalIns();
		}
	};

	class LirStorage;

	// concrete
	class LirReader : public LirFilter
	{
		LirStorage*                             _in;
		IntrusiveLinkedListReverseIterator      _it;
		LIns*                                   _finalIns;

	public:
		LirReader(LirStorage* in);
		virtual ~LirReader();

		// Returns next instruction and advances to the prior instruction.
		// Invariant: never returns a skip.
		virtual LIns* read();

		// Returns the instruction that read() will return on the next call.
		// Invariant: never returns a skip.
		LIns* peek();

		virtual LIns* finalIns() {
			return _finalIns;
		}
	};

	verbose_only(void live(LirFilter* in, Allocator* alloc, LInsPrinter* printer, LogControl*);)


#ifdef NANOJIT_VERBOSE
	/* A listing filter for LIR, going through backwards.  It merely
	   passes its input to its output, but notes it down too.  When
	   finish() is called, prints out what went through.  Is intended to be
	   used to print arbitrary intermediate transformation stages of
	   LIR. */
	class ReverseLister : public LirFilter
	{
		Allocator*   _alloc;
		LInsPrinter* _printer;
		const char*  _title;
		StringList   _strs;
		LogControl*  _logc;
		LIns*        _prevIns;
	public:
		ReverseLister(LirFilter* in, Allocator* alloc,
					  LInsPrinter* printer, LogControl* logc, const char* title)
			: LirFilter(in)
			, _alloc(alloc)
			, _printer(printer)
			, _title(title)
			, _strs(alloc)
			, _logc(logc)
			, _prevIns(NULL)
		{ }

		void finish();
		LIns* read();
	};

#endif


#ifdef NANOJIT_DEBUG
	// This just checks things that aren't possible to check in
	// ValidateWriter, eg. whether all branch targets are set and are labels.
	class ValidateReader: public LirFilter {
	public:
		ValidateReader(LirFilter* in);
		virtual LIns* read();
	};
#endif
}

#endif // __nanojit_LirFilter__

