

#if !defined(__nanojit_LInsStorage__)
#define __nanojit_LInsStorage__

#include "utils/IntrusiveLinkedList.h"
#include <stdint.h>

namespace nanojit
{

struct LInsElementStorage
{
	IntrusiveListNode   node;
	LIns                data;
};

class LInsElement
{
public:
	static const size_t         NODE_OFFSET = offsetof(LInsElementStorage, node);
	static const size_t         DATA_OFFSET = offsetof(LInsElementStorage, data);

private:
	LInsElementStorage          storage;

public:
	LInsElement()
	{

	}

	~LInsElement()
	{

	}

	NANOJIT_FORCEINLINE LIns* data()
	{
		return &storage.data;
	}

	NANOJIT_FORCEINLINE IntrusiveListNode* node()
	{
		return &storage.node;
	}


	static NANOJIT_FORCEINLINE LInsElement* getElement(LIns* value)
	{
		return reinterpret_cast<LInsElement*>(reinterpret_cast<uint8_t*>(value) - DATA_OFFSET);
	}

private:
	// forbid copy
	LInsElement(const LInsElement&);
	LInsElement& operator=(const LInsElement& a_oOther);
};


class LirStorage
{

private:
	static const unsigned int       SEGMENT_STORAGE_SIZE = 2048;

	struct StorageSegment
	{
		StorageSegment*         next;
		LInsElement             storage[SEGMENT_STORAGE_SIZE];
	};

	Allocator*                      _allocator;
	StorageSegment*                 _head;
	StorageSegment*                 _current;
	unsigned                        _storageElementIndex;
	IntrusiveLinkedList             _contentList;


public:
	typedef IntrusiveLinkedList::Iterator           Iterator;
	typedef IntrusiveLinkedList::ReverseIterator    ReverseIterator;

public: 
								LirStorage();
								~LirStorage();

	void                        initialize(Allocator* allocator);
	void                        shutdown();

	void                        clear();

	LInsElement*                insertNewElement();

	// return the element returned by the next call to insertNewElement
	LInsElement*                peekNextElement();


	void                        moveBefore(LInsElement* elementToMove, LInsElement* elementTarget);
	void                        moveAfter(LInsElement* elementToMove, LInsElement* elementTarget);
	void                        remove(LInsElement* elementToRemove);

	Iterator                    getBegin();
	Iterator                    getEnd();
	Iterator                    getIteratorAt(LInsElement* element);


	ReverseIterator             getReverseBegin();
	ReverseIterator             getReverseEnd();
	ReverseIterator             getReverseIteratorAt(LInsElement* element);




private:
	void                                    createNewSegment();
	LInsElement*                            getNewElement();
	

};


inline LirStorage::Iterator LirStorage::getBegin()
{
	return _contentList.getBegin();
}

inline LirStorage::Iterator LirStorage::getEnd()
{
	return _contentList.getEnd();
}

inline LirStorage::Iterator LirStorage::getIteratorAt(LInsElement* element)
{
	return _contentList.getIteratorAt(element);
}


inline LirStorage::ReverseIterator LirStorage::getReverseBegin()
{
	return _contentList.getReverseBegin();
}

inline LirStorage::ReverseIterator LirStorage::getReverseEnd()
{
	return _contentList.getReverseEnd();
}

inline LirStorage::ReverseIterator LirStorage::getReverseIteratorAt(LInsElement* element)
{
	return _contentList.getReverseIteratorAt(element);
}

inline void LirStorage::remove(LInsElement* elementToRemove)
{
	return _contentList.remove(_contentList.getIteratorAt(elementToRemove));
}


} // namespace nanojit

#endif // __nanojit_LInsStorage__


