
#ifndef __nanojit_portability_h__
#define __nanojit_portability_h__

//
#include <stdint.h>
#include <string.h>
#include <math.h>
#include <ctype.h>


#if defined NANOJIT_64BIT
#define IF_64BIT(...) __VA_ARGS__
#define UNLESS_64BIT(...)
#define CASE32(x)
#define CASE64(x)   case x
#else
#define IF_64BIT(...)
#define UNLESS_64BIT(...) __VA_ARGS__
#define CASE32(x)   case x
#define CASE64(x)
#endif

#if defined NANOJIT_HOST_PLATFORM_X64
#define CASE86(x)   case x
#else
#define CASE86(x)
#endif

#if defined(__ghs__)
#define NANOJIT_FORCEINLINE __attribute__((always_inline))
#elif defined(_MSC_VER)
#define NANOJIT_FORCEINLINE __forceinline
#else
#define NANOJIT_FORCEINLINE inline
#endif

#ifdef _MSC_VER
#define NANOJIT_ALIGN16(type) __declspec(align(16)) type
#else
#define NANOJIT_ALIGN16(type) type __attribute__ ((aligned (16)))
#endif


#define VMPI_abort abort
#define VMPI_strlen strlen
#define VMPI_strcat strcat
#define VMPI_strcmp strcmp
#define VMPI_strncat strncat
#define VMPI_strcpy strcpy
#define VMPI_sprintf sprintf
#ifdef _MSC_VER
#   define VMPI_snprintf sprintf_s
#else
#   define VMPI_snprintf snprintf
#endif
#define VMPI_vfprintf vfprintf
#define VMPI_memset memset
#define VMPI_memcmp memcmp
#define VMPI_memcpy memcpy
#define VMPI_isdigit isdigit
#define VMPI_getDate()


namespace nanojit
{
namespace debug
{

void Log(const char* a_sMessage, ...);
void AssertFail(const char* a_sMessage);
void Break();

} // namespace debug
} // namespace nanojit


#endif // #ifndef __nanojit_portability_h__

