/* -*- Mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4 -*- */
/* vi: set ts=4 sw=4 expandtab: (add to ~/.vimrc: set modeline modelines=5) */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#ifndef __nanojit_Native__
#define __nanojit_Native__

// define PEDANTIC=1 to ignore specialized forms, force general forms
// for everything, far branches, extra page-linking, etc.  This will
// flush out many corner cases.

#define NANOJIT_PEDANTIC 0
#if NANOJIT_PEDANTIC
#  define UNLESS_PEDANTIC(...)
#  define IF_PEDANTIC(...) __VA_ARGS__
#else
#  define UNLESS_PEDANTIC(...) __VA_ARGS__
#  define IF_PEDANTIC(...)
#endif

#if defined(NANOJIT_HOST_PLATFORM_ARM)
#include "NativeARM.h"
#elif defined(NANOJIT_HOST_PLATFORM_ARM64)
#include "NativeARM64.h"
#elif defined(NANOJIT_HOST_PLATFORM_PPC)
#include "NativePPC.h"
#elif defined(NANOJIT_HOST_PLATFORM_X64)
#include "NativeX64.h"
#elif defined(NANOJIT_HOST_PLATFORM_INTERPRETER)
#include "NativeInterpreter.h"
#elif defined(NANOJIT_HOST_PLATFORM_DUMMY)
#include "NativeDummy.h"
#else
#error "unknown nanojit target architecture"
#endif

#ifndef NANOJIT_USES_IMMD_POOL
#  define NANOJIT_USES_IMMD_POOL 0
#endif

	#if defined(NANOJIT_VERBOSE)
		inline char cvaltoa(unsigned char u) {
			return u<10 ? u+'0' : u+'a'-10;
		}

		inline char* appendHexVals(char* str, char* valFrom, char* valTo) {
			NanoAssert(valFrom <= valTo);
			str += VMPI_strlen(str);
			for(char* ch = valFrom; ch < valTo; ch++) {
				unsigned char u = (unsigned char)*ch;
				*str++ = cvaltoa(u >> 4);
				*str++ = cvaltoa(u &  0xf);
				*str++ = ' ';
			}
			*str = '\0';
			return str;
		}

		inline char* padTo(char* str, int n, char c=' ') {
			char* start = str + VMPI_strlen(str);
			char* end = &str[n];
			while(start < end)
				*start++ = c;
			*end = '\0';
			return end;
		}

		// Used for printing native instructions.  Like Assembler::outputf(),
		// but only outputs if LC_Native is set.  Also prepends the output
		// with the address of the current native instruction.
		#define asm_output(...) do {                                            \
			if (_logc->getLcBits() & LC_Native) {                               \
				maybe_disassemble();                                            \
				outline[0]='\0';                                                \
				VMPI_sprintf(outline, "%p  ", _nIns);                           \
				if (_logc->getLcBits() & LC_Bytes) {                            \
					appendHexVals(outline, (char*)_nIns, (char*)_nInsAfter);    \
					padTo(outline, 3*15);                                       \
				}                                                               \
				VMPI_sprintf(outline + VMPI_strlen(outline), ##__VA_ARGS__);    \
				output();                                                       \
				_nInsAfter = _nIns;                                             \
			}                                                                   \
		} while (0) /* no semi */
		#define gpn(r)                  regNames[(REGNUM(r))]
		#define verbose_val(v) (v)
	#else
		#define asm_output(...)
		#define gpn(r)                  "reg"
		#define verbose_val(v) 0
	#endif /* NANOJIT_VERBOSE */

#endif // __nanojit_Native__
